/*
 * Copyright (C) 2013 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "APIContentRuleList.h"
#include "APIDictionary.h"
#include "WKAPICast.h"
#include "WKArray.h"
#include "WKContextPrivate.h"
#include "WKMutableDictionary.h"
#include "WKPageGroup.h"
#include "WKPreferencesRefPrivate.h"
#include "WebPageGroup.h"
#include "WebUserContentControllerProxy.h"

#if PLATFORM(MAC)
#include "WKContextPrivateMac.h"
#endif

// Deprecated functions that should be removed from the framework once nobody uses them.

using namespace WebKit;

void WKContextSetUsesNetworkProcess(WKContextRef, bool)
{
}

void WKContextSetProcessModel(WKContextRef, WKProcessModel)
{
}

WKStringRef WKPageGroupCopyIdentifier(WKPageGroupRef)
{
    return nullptr;
}

void WKPageGroupAddUserContentFilter(WKPageGroupRef pageGroupRef, WKUserContentFilterRef contentFilterRef)
{
#if ENABLE(CONTENT_EXTENSIONS)
    toImpl(pageGroupRef)->userContentController().addContentRuleList(*toImpl(contentFilterRef));
#else
    UNUSED_PARAM(pageGroupRef);
    UNUSED_PARAM(contentFilterRef);
#endif
}

void WKPageGroupRemoveUserContentFilter(WKPageGroupRef pageGroupRef, WKStringRef contentFilterNameRef)
{
#if ENABLE(CONTENT_EXTENSIONS)
    toImpl(pageGroupRef)->userContentController().removeContentRuleList(toWTFString(contentFilterNameRef));
#else
    UNUSED_PARAM(pageGroupRef);
    UNUSED_PARAM(contentFilterNameRef);
#endif
}
