/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef PluginProcessCreationParameters_h
#define PluginProcessCreationParameters_h

#if ENABLE(NETSCAPE_PLUGIN_API)

#include "Attachment.h"
#include "PluginProcessAttributes.h"
#include <wtf/Seconds.h>

#if PLATFORM(COCOA)
#include <WebCore/MachSendRight.h>
#endif

namespace IPC {
class Decoder;
class Encoder;
}

namespace WebKit {

struct PluginProcessCreationParameters {
    PluginProcessCreationParameters();

    void encode(IPC::Encoder&) const;
    static bool decode(IPC::Decoder&, PluginProcessCreationParameters&);

    PluginProcessType processType;
    bool supportsAsynchronousPluginInitialization;

    Seconds minimumLifetime;
    Seconds terminationTimeout;

#if PLATFORM(COCOA)
    WebCore::MachSendRight acceleratedCompositingPort;
    RetainPtr<CFDataRef> networkATSContext;
#endif
#if OS(LINUX)
    IPC::Attachment memoryPressureMonitorHandle;
#endif
};

} // namespace WebKit

#endif // ENABLE(NETSCAPE_PLUGIN_API)

#endif // PluginProcessCreationParameters_h
