/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "StorageProcessProxy.h"

#if ENABLE(SANDBOX_EXTENSIONS)
#include "ArgumentCoders.h"
#endif
#include "Attachment.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "StorageProcessProxyMessages.h"
#include "WebsiteData.h"
#if ENABLE(SANDBOX_EXTENSIONS)
#include <wtf/Vector.h>
#endif
#if ENABLE(SANDBOX_EXTENSIONS)
#include <wtf/text/WTFString.h>
#endif

namespace WebKit {

void StorageProcessProxy::didReceiveStorageProcessProxyMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::StorageProcessProxy::DidCreateStorageToWebProcessConnection::name()) {
        IPC::handleMessage<Messages::StorageProcessProxy::DidCreateStorageToWebProcessConnection>(decoder, this, &StorageProcessProxy::didCreateStorageToWebProcessConnection);
        return;
    }
    if (decoder.messageName() == Messages::StorageProcessProxy::DidFetchWebsiteData::name()) {
        IPC::handleMessage<Messages::StorageProcessProxy::DidFetchWebsiteData>(decoder, this, &StorageProcessProxy::didFetchWebsiteData);
        return;
    }
    if (decoder.messageName() == Messages::StorageProcessProxy::DidDeleteWebsiteData::name()) {
        IPC::handleMessage<Messages::StorageProcessProxy::DidDeleteWebsiteData>(decoder, this, &StorageProcessProxy::didDeleteWebsiteData);
        return;
    }
    if (decoder.messageName() == Messages::StorageProcessProxy::DidDeleteWebsiteDataForOrigins::name()) {
        IPC::handleMessage<Messages::StorageProcessProxy::DidDeleteWebsiteDataForOrigins>(decoder, this, &StorageProcessProxy::didDeleteWebsiteDataForOrigins);
        return;
    }
#if ENABLE(SANDBOX_EXTENSIONS)
    if (decoder.messageName() == Messages::StorageProcessProxy::GetSandboxExtensionsForBlobFiles::name()) {
        IPC::handleMessage<Messages::StorageProcessProxy::GetSandboxExtensionsForBlobFiles>(decoder, this, &StorageProcessProxy::getSandboxExtensionsForBlobFiles);
        return;
    }
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit
