/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "WebSocketStream.h"

#include "DataReference.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "WebCoreArgumentCoders.h"
#include "WebSocketStreamMessages.h"
#include <WebCore/SocketStreamError.h>

namespace WebKit {

void WebSocketStream::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::WebSocketStream::DidOpenSocketStream::name()) {
        IPC::handleMessage<Messages::WebSocketStream::DidOpenSocketStream>(decoder, this, &WebSocketStream::didOpenSocketStream);
        return;
    }
    if (decoder.messageName() == Messages::WebSocketStream::DidCloseSocketStream::name()) {
        IPC::handleMessage<Messages::WebSocketStream::DidCloseSocketStream>(decoder, this, &WebSocketStream::didCloseSocketStream);
        return;
    }
    if (decoder.messageName() == Messages::WebSocketStream::DidReceiveSocketStreamData::name()) {
        IPC::handleMessage<Messages::WebSocketStream::DidReceiveSocketStreamData>(decoder, this, &WebSocketStream::didReceiveSocketStreamData);
        return;
    }
    if (decoder.messageName() == Messages::WebSocketStream::DidFailToReceiveSocketStreamData::name()) {
        IPC::handleMessage<Messages::WebSocketStream::DidFailToReceiveSocketStreamData>(decoder, this, &WebSocketStream::didFailToReceiveSocketStreamData);
        return;
    }
    if (decoder.messageName() == Messages::WebSocketStream::DidUpdateBufferedAmount::name()) {
        IPC::handleMessage<Messages::WebSocketStream::DidUpdateBufferedAmount>(decoder, this, &WebSocketStream::didUpdateBufferedAmount);
        return;
    }
    if (decoder.messageName() == Messages::WebSocketStream::DidFailSocketStream::name()) {
        IPC::handleMessage<Messages::WebSocketStream::DidFailSocketStream>(decoder, this, &WebSocketStream::didFailSocketStream);
        return;
    }
    if (decoder.messageName() == Messages::WebSocketStream::DidSendData::name()) {
        IPC::handleMessage<Messages::WebSocketStream::DidSendData>(decoder, this, &WebSocketStream::didSendData);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit
